@@ -19,7 +19,6 @@ import zmq
 from zmq.ssh import tunnel as zmqtunnel
 
 from spyder_kernels.comms.commbase import CommBase, CommError
-from spyder.py3compat import TimeoutError
 from spyder.plugins.ipythonconsole.utils.ssh import openssh_tunnel
 
 logger = logging.getLogger(__name__)
@@ -84,10 +83,19 @@ class KernelComm(CommBase, QObject):
 
     def shutdown_comm_channel(self):
         """Shutdown the comm channel."""
-        channel = self.kernel_client.comm_channel
+        # This is necessary to avoid an error when closing the console.
+        # Fixes spyder-ide/spyder#15645
+        try:
+            channel = self.kernel_client.comm_channel
+        except AttributeError:
+            channel = None
+
         if channel:
-            msg = self.kernel_client.session.msg('shutdown_request', {})
-            channel.send(msg)
+            id_list = self.get_comm_id_list()
+            for comm_id in id_list:
+                msg = self.kernel_client.session.msg(
+                    'shutdown_request', {"comm_id": comm_id})
+                channel.send(msg)
             self.kernel_client.comm_channel = None
 
     def comm_channel_connected(self):
@@ -114,6 +122,7 @@ class KernelComm(CommBase, QObject):
         try:
             yield
         finally:
+            id_list = self.get_comm_id_list(comm_id)
             for comm_id in id_list:
                 self._comms[comm_id]['comm']._send_channel = (
                     self.kernel_client.shell_channel)
@@ -127,19 +136,40 @@ class KernelComm(CommBase, QObject):
                 call_dict, data, is_error)
 
     def remove(self, comm_id=None):
-        """Remove the comm without notifying the other side."""
+        """
+        Remove the comm without notifying the other side.
+
+        Use when the other side is already down.
+        """
         id_list = self.get_comm_id_list(comm_id)
         for comm_id in id_list:
             del self._comms[comm_id]
 
+    def close(self, comm_id=None, shutdown_channel=True):
+        """Ask kernel to close comm and send confirmation."""
+        if shutdown_channel:
+            # Only shutdown the channel if the kernel closes as well
+            self.shutdown_comm_channel()
+        id_list = self.get_comm_id_list(comm_id)
+        for comm_id in id_list:
+            # Send comm_close directly to avoid really closing the comm
+            self._comms[comm_id]['comm']._send_msg(
+                'comm_close', {}, None, None, None)
+
     def open_comm(self, kernel_client):
         """Open comm through the kernel client."""
         self.kernel_client = kernel_client
         self.kernel_client.comm_channel = None
-        self._register_comm(
-            # Create new comm and send the highest protocol
-            kernel_client.comm_manager.new_comm(self._comm_name, data={
-                'pickle_protocol': pickle.HIGHEST_PROTOCOL}))
+        try:
+            self._register_comm(
+                # Create new comm and send the highest protocol
+                kernel_client.comm_manager.new_comm(self._comm_name, data={
+                    'pickle_protocol': pickle.HIGHEST_PROTOCOL}))
+        except AttributeError:
+            logger.info(
+                "Unable to open comm due to unexistent comm manager: " +
+                "kernel_client.comm_manager=" + str(kernel_client.comm_manager)
+            )
 
     def remote_call(self, interrupt=False, blocking=False, callback=None,
                     comm_id=None, timeout=None, display_error=False):
@@ -168,7 +198,7 @@ class KernelComm(CommBase, QObject):
             else:
                 # The user has other problems
                 logger.info(
-                    "Dropping message because kernel is dead: ",
+                    "Dropping message because kernel is dead: %s",
                     str(call_dict)
                 )
                 return
@@ -230,8 +260,8 @@ class KernelComm(CommBase, QObject):
             raise RuntimeError("Kernel is dead")
 
         # Create event loop to wait with
-        wait_loop = QEventLoop()
-        wait_timeout = QTimer()
+        wait_loop = QEventLoop(None)
+        wait_timeout = QTimer(self)
         wait_timeout.setSingleShot(True)
 
         # Connect signals to stop kernel loop
