@@ -9,18 +9,21 @@ Layout Plugin.
 """
 # Standard library imports
 import configparser as cp
+import logging
 import os
 
 # Third party imports
 from qtpy.QtCore import Qt, QByteArray, QSize, QPoint, Slot
-from qtpy.QtWidgets import QApplication, QDesktopWidget, QDockWidget
+from qtpy.QtWidgets import QApplication, QDesktopWidget
 
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
-from spyder.api.plugins import Plugins, SpyderPluginV2
+from spyder.api.plugins import (
+    Plugins, DockablePlugins, SpyderDockablePlugin, SpyderPluginV2)
 from spyder.api.plugin_registration.decorators import (
     on_plugin_available, on_plugin_teardown)
-from spyder.api.translations import get_translation
+from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
+from spyder.api.translations import _
 from spyder.api.utils import get_class_values
 from spyder.plugins.mainmenu.api import ApplicationMenus, ViewMenuSections
 from spyder.plugins.layout.container import (
@@ -35,10 +38,8 @@ from spyder.plugins.toolbar.api import (
 from spyder.py3compat import qbytearray_to_str  # FIXME:
 
 
-# Localization
-_ = get_translation("spyder")
-
-# Constants
+# For logging
+logger = logging.getLogger(__name__)
 
 # Number of default layouts available
 DEFAULT_LAYOUTS = get_class_values(DefaultLayouts)
@@ -68,13 +69,13 @@ class Layout(SpyderPluginV2):
     """
     NAME = "layout"
     CONF_SECTION = "quick_layouts"
-    REQUIRES = [Plugins.All]  # Uses wildcard to require all the plugins
+    REQUIRES = [Plugins.All]  # Uses wildcard to require all plugins
     CONF_FILE = False
     CONTAINER_CLASS = LayoutContainer
     CAN_BE_DISABLED = False
 
-    # --- SpyderDockablePlugin API
-    # ------------------------------------------------------------------------
+    # ---- SpyderDockablePlugin API
+    # -------------------------------------------------------------------------
     @staticmethod
     def get_name():
         return _("Layout")
@@ -196,13 +197,52 @@ class Layout(SpyderPluginV2):
         self.setup_layout(default=False)
 
     def on_mainwindow_visible(self):
-        # Populate panes menu
+        # Populate `Panes > View` menu.
+        # This **MUST** be done before restoring the last visible plugins, so
+        # that works as expected.
         self.create_plugins_menu()
+
+        # Restore last visible plugins.
+        # This **MUST** be done before running on_mainwindow_visible for the
+        # other plugins so that the user doesn't experience sudden jumps in the
+        # interface.
+        self.restore_visible_plugins()
+
         # Update panes and toolbars lock status
         self.toggle_lock(self._interface_locked)
 
-    # --- Plubic API
-    # ------------------------------------------------------------------------
+    # ---- Private API
+    # -------------------------------------------------------------------------
+    def _get_internal_dockable_plugins(self):
+        """Get the list of internal dockable plugins"""
+        return get_class_values(DockablePlugins)
+
+    def _update_fullscreen_action(self):
+        if self._fullscreen_flag:
+            icon = self.create_icon('window_nofullscreen')
+        else:
+            icon = self.create_icon('window_fullscreen')
+        self.get_container()._fullscreen_action.setIcon(icon)
+
+    def _update_lock_interface_action(self):
+        """
+        Helper method to update the locking of panes/dockwidgets and toolbars.
+
+        Returns
+        -------
+        None.
+        """
+        if self._interface_locked:
+            icon = self.create_icon('drag_dock_widget')
+            text = _('Unlock panes and toolbars')
+        else:
+            icon = self.create_icon('lock')
+            text = _('Lock panes and toolbars')
+        self.lock_interface_action.setIcon(icon)
+        self.lock_interface_action.setText(text)
+
+    # ---- Helper methods
+    # -------------------------------------------------------------------------
     def get_last_plugin(self):
         """
         Return the last focused dockable plugin.
@@ -227,6 +267,8 @@ class Layout(SpyderPluginV2):
         """
         return self._fullscreen_flag
 
+    # ---- Layout handling
+    # -------------------------------------------------------------------------
     def register_layout(self, parent_plugin, layout_type):
         """
         Register a new layout type.
@@ -240,6 +282,21 @@ class Layout(SpyderPluginV2):
         """
         self.get_container().register_layout(parent_plugin, layout_type)
 
+    def register_custom_layouts(self):
+        """Register custom layouts provided by external plugins."""
+        for plugin_name in PLUGIN_REGISTRY.external_plugins:
+            plugin_instance = self.get_plugin(plugin_name)
+            if hasattr(plugin_instance, 'CUSTOM_LAYOUTS'):
+                if isinstance(plugin_instance.CUSTOM_LAYOUTS, list):
+                    for custom_layout in plugin_instance.CUSTOM_LAYOUTS:
+                        self.register_layout(self, custom_layout)
+                else:
+                    logger.info(
+                        f'Unable to load custom layouts for plugin '
+                        f'{plugin_name}. Expecting a list of layout classes '
+                        f'but got {plugin_instance.CUSTOM_LAYOUTS}.'
+                    )
+
     def get_layout(self, layout_id):
         """
         Get a registered layout by his ID.
@@ -575,6 +632,8 @@ class Layout(SpyderPluginV2):
             section=section,
         )
 
+    # ---- Maximize, close, switch to dockwidgets/plugins
+    # -------------------------------------------------------------------------
     @Slot()
     def close_current_dockwidget(self):
         """Search for the currently focused plugin and close it."""
@@ -641,10 +700,11 @@ class Layout(SpyderPluginV2):
             try:
                 # New API
                 self.main.setCentralWidget(self._last_plugin.get_widget())
+                self._last_plugin.get_widget().set_maximized_state(True)
             except AttributeError:
                 # Old API
                 self.main.setCentralWidget(self._last_plugin)
-            self._last_plugin._ismaximized = True
+                self._last_plugin._ismaximized = True
 
             # Workaround to solve an issue with editor's outline explorer:
             # (otherwise the whole plugin is hidden and so is the outline
@@ -679,7 +739,7 @@ class Layout(SpyderPluginV2):
 
             try:
                 # New API
-                self._last_plugin.get_widget().is_maximized = False
+                self._last_plugin.get_widget().set_maximized_state(False)
             except AttributeError:
                 # Old API
                 self._last_plugin._ismaximized = False
@@ -695,13 +755,90 @@ class Layout(SpyderPluginV2):
                 # Old API
                 self._last_plugin.get_focus_widget().setFocus()
 
-    def _update_fullscreen_action(self):
-        if self._fullscreen_flag:
-            icon = self.create_icon('window_nofullscreen')
-        else:
-            icon = self.create_icon('window_fullscreen')
-        self.get_container()._fullscreen_action.setIcon(icon)
+    def unmaximize_dockwidget(self):
+        """Unmaximize any dockable plugin."""
+        if self.maximize_action.isChecked():
+            self.maximize_action.setChecked(False)
 
+    def unmaximize_other_dockwidget(self, plugin_instance):
+        """
+        Unmaximize the currently maximized plugin, if not `plugin_instance`.
+        """
+        last_plugin = self.get_last_plugin()
+        is_maximized = False
+
+        if last_plugin is not None:
+            try:
+                # New API
+                is_maximized = (
+                    last_plugin.get_widget().get_maximized_state()
+                )
+            except AttributeError:
+                # Old API
+                is_maximized = last_plugin._ismaximized
+
+        if (
+            last_plugin is not None
+            and is_maximized
+            and last_plugin is not plugin_instance
+        ):
+            self.unmaximize_dockwidget()
+
+    def switch_to_plugin(self, plugin, force_focus=None):
+        """
+        Switch to `plugin`.
+
+        Notes
+        -----
+        This operation unmaximizes the current plugin (if any), raises
+        this plugin to view (if it's hidden) and gives it focus (if
+        possible).
+        """
+        last_plugin = self.get_last_plugin()
+
+        try:
+            # New API
+            if (
+                last_plugin is not None
+                and last_plugin.get_widget().get_maximized_state()
+                and last_plugin is not plugin
+            ):
+                if self.maximize_action.isChecked():
+                    self.maximize_action.setChecked(False)
+                else:
+                    self.maximize_action.setChecked(True)
+        except AttributeError:
+            # Old API
+            if (
+                last_plugin is not None
+                and last_plugin._ismaximized
+                and last_plugin is not plugin
+            ):
+                if self.maximize_action.isChecked():
+                    self.maximize_action.setChecked(False)
+                else:
+                    self.maximize_action.setChecked(True)
+
+        try:
+            # New API
+            if not plugin.toggle_view_action.isChecked():
+                plugin.toggle_view_action.setChecked(True)
+                plugin.get_widget().is_visible = False
+        except AttributeError:
+            # Old API
+            if not plugin._toggle_view_action.isChecked():
+                plugin._toggle_view_action.setChecked(True)
+                plugin._widget._is_visible = False
+
+        try:
+            # New API
+            plugin.change_visibility(True, force_focus=force_focus)
+        except AttributeError:
+            # Old API
+            plugin._visibility_changed(True)
+
+    # ---- Menus and actions
+    # -------------------------------------------------------------------------
     @Slot()
     def toggle_fullscreen(self):
         """
@@ -768,7 +905,13 @@ class Layout(SpyderPluginV2):
                 action.action_id = f'switch to {plugin.CONF_SECTION}'
 
             if action:
-                action.setChecked(plugin.dockwidget.isVisible())
+                # Plugins that fail their compatibility checks don't have a
+                # dockwidget. So, we need to skip them from the plugins menu.
+                # Fixes spyder-ide/spyder#21074
+                if plugin.dockwidget is None:
+                    continue
+                else:
+                    action.setChecked(plugin.dockwidget.isVisible())
 
             try:
                 name = plugin.CONF_SECTION
@@ -790,23 +933,6 @@ class Layout(SpyderPluginV2):
     def lock_interface_action(self):
         return self.get_container()._lock_interface_action
 
-    def _update_lock_interface_action(self):
-        """
-        Helper method to update the locking of panes/dockwidgets and toolbars.
-
-        Returns
-        -------
-        None.
-        """
-        if self._interface_locked:
-            icon = self.create_icon('drag_dock_widget')
-            text = _('Unlock panes and toolbars')
-        else:
-            icon = self.create_icon('lock')
-            text = _('Lock panes and toolbars')
-        self.lock_interface_action.setIcon(icon)
-        self.lock_interface_action.setText(text)
-
     def toggle_lock(self, value=None):
         """Lock/Unlock dockwidgets and toolbars."""
         self._interface_locked = (
@@ -815,6 +941,12 @@ class Layout(SpyderPluginV2):
         self._update_lock_interface_action()
         # Apply lock to panes
         for plugin in self.get_dockable_plugins():
+            # Plugins that fail their compatibility checks don't have a
+            # dockwidget. So, we need to skip them from the code below.
+            # Fixes spyder-ide/spyder#21074
+            if plugin.dockwidget is None:
+                continue
+
             if self._interface_locked:
                 if plugin.dockwidget.isFloating():
                     plugin.dockwidget.setFloating(False)
@@ -827,3 +959,173 @@ class Layout(SpyderPluginV2):
         toolbar = self.get_plugin(Plugins.Toolbar)
         if toolbar:
             toolbar.toggle_lock(value=self._interface_locked)
+
+    # ---- Visible dockable plugins
+    # -------------------------------------------------------------------------
+    def restore_visible_plugins(self):
+        """
+        Restore dockable plugins that were visible during the previous session.
+        """
+        logger.info("Restoring visible plugins from the previous session")
+        visible_plugins = self.get_conf('last_visible_plugins', default=[])
+
+        # This should only be necessary the first time this method is run
+        if not visible_plugins:
+            visible_plugins = [Plugins.IPythonConsole, Plugins.Help,
+                               Plugins.Editor]
+
+        # Restore visible plugins
+        for plugin in visible_plugins:
+            plugin_class = self.get_plugin(plugin, error=False)
+            if (
+                plugin_class
+                # This check is necessary for spyder-ide/spyder#21074
+                and plugin_class.dockwidget is not None
+                and plugin_class.dockwidget.isVisible()
+            ):
+                plugin_class.dockwidget.raise_()
+
+    def save_visible_plugins(self):
+        """Save visible plugins."""
+        logger.debug("Saving visible plugins to config system")
+
+        visible_plugins = []
+        for plugin in self.get_dockable_plugins():
+            try:
+                # New API
+                if plugin.get_widget().is_visible:
+                    visible_plugins.append(plugin.NAME)
+            except AttributeError:
+                # Old API
+                if plugin._isvisible:
+                    visible_plugins.append(plugin.NAME)
+
+        self.set_conf('last_visible_plugins', visible_plugins)
+
+    # ---- Tabify plugins
+    # -------------------------------------------------------------------------
+    def tabify_plugins(self, first, second):
+        """Tabify plugin dockwigdets."""
+        self.main.tabifyDockWidget(first.dockwidget, second.dockwidget)
+
+    def tabify_plugin(self, plugin, default=None):
+        """
+        Tabify `plugin` using the list of possible TABIFY options.
+
+        Only do this if the dockwidget does not have more dockwidgets
+        in the same position and if the plugin is using the New API.
+        """
+        def tabify_helper(plugin, next_to_plugins):
+            for next_to_plugin in next_to_plugins:
+                try:
+                    self.tabify_plugins(next_to_plugin, plugin)
+                    break
+                except SpyderAPIError as err:
+                    logger.error(err)
+
+        # If TABIFY not defined use the [default]
+        tabify = getattr(plugin, 'TABIFY', [default])
+        if not isinstance(tabify, list):
+            next_to_plugins = [tabify]
+        else:
+            next_to_plugins = tabify
+
+        # Check if TABIFY is not a list with None as unique value or a default
+        # list
+        if tabify in [[None], []]:
+            return False
+
+        # Get the actual plugins from their names
+        next_to_plugins = [self.get_plugin(p) for p in next_to_plugins]
+
+        if plugin.get_conf('first_time', True):
+            # This tabifies external and internal plugins that are loaded for
+            # the first time, and internal ones that are not part of the
+            # default layout.
+            if (
+                isinstance(plugin, SpyderDockablePlugin)
+                and plugin.NAME != Plugins.Console
+            ):
+                logger.info(
+                    f"Tabifying {plugin.NAME} plugin for the first time next "
+                    f"to {next_to_plugins}"
+                )
+                tabify_helper(plugin, next_to_plugins)
+
+                # Show external plugins
+                if plugin.NAME in PLUGIN_REGISTRY.external_plugins:
+                    plugin.get_widget().toggle_view(True)
+
+            plugin.set_conf('enable', True)
+            plugin.set_conf('first_time', False)
+        else:
+            # This is needed to ensure that, when switching to a different
+            # layout, any plugin (external or internal) not part of its
+            # declared areas is tabified as expected.
+            # Note: Check if `plugin` has no other dockwidgets in the same
+            # position before proceeding.
+            if not bool(self.main.tabifiedDockWidgets(plugin.dockwidget)):
+                logger.info(f"Tabifying {plugin.NAME} plugin")
+                tabify_helper(plugin, next_to_plugins)
+
+        return True
+
+    def tabify_new_plugins(self):
+        """
+        Tabify new dockable plugins, i.e. plugins that were not part of the
+        interface in the last session.
+
+        Notes
+        -----
+        This is only necessary the first time a plugin is loaded. Afterwards,
+        the plugin's placement is recorded in the window hexstate, which is
+        loaded in the next session.
+        """
+        # Detect if a new dockable internal plugin hasn't been added to the
+        # DockablePlugins enum and raise an error if that's the case.
+        for plugin in self.get_dockable_plugins():
+            if (
+                plugin.NAME in PLUGIN_REGISTRY.internal_plugins
+                and plugin.NAME not in self._get_internal_dockable_plugins()
+            ):
+                raise SpyderAPIError(
+                    f"Plugin {plugin.NAME} is a new dockable plugin but it "
+                    f"hasn't been added to the DockablePlugins enum. Please "
+                    f"do that to avoid this error."
+                )
+
+        # If this is the first time Spyder runs, then we don't need to go
+        # beyond this point because all plugins are tabified in the
+        # set_main_window_layout method of any layout.
+        if self._first_spyder_run:
+            # Save the list of internal dockable plugins to compare it with
+            # the current ones during the next session.
+            self.set_conf(
+                'internal_dockable_plugins',
+                self._get_internal_dockable_plugins()
+            )
+            return
+
+        logger.debug("Tabifying new plugins")
+
+        # Get the list of internal dockable plugins that were present in the
+        # last session to decide which ones need to be tabified.
+        last_internal_dockable_plugins = self.get_conf(
+            'internal_dockable_plugins',
+            default=self._get_internal_dockable_plugins()
+        )
+
+        # Tabify new internal plugins
+        for plugin_name in self._get_internal_dockable_plugins():
+            if plugin_name not in last_internal_dockable_plugins:
+                plugin = self.get_plugin(plugin_name, error=False)
+                if plugin:
+                    self.tabify_plugin(plugin, Plugins.Console)
+
+        # Tabify new external plugins
+        for plugin in self.get_dockable_plugins():
+            if (
+                plugin.NAME in PLUGIN_REGISTRY.external_plugins
+                and plugin.get_conf('first_time', True)
+            ):
+                self.tabify_plugin(plugin, Plugins.Console)
